@@ -7,12 +7,9 @@
 ##' user-friendly front-end to this function. Details about the priors can be 
 ##' found in the help for \code{\link{anovaBF}} and the references therein.
 ##' 
-##' Arguments \code{struc} and \code{gMap} provide a way of grouping columns of 
+##' Argument \code{gMap} provides a way of grouping columns of 
 ##' the design matrix as a factor; the effects in each group will share a common
-##' \eqn{g} parameter. Only one of these arguments is needed; if both are given,
-##' \code{gMap} takes precedence.
-##' 
-##' \code{gMap} should be a vector of the same length as the number of 
+##' \eqn{g} parameter. \code{gMap} should be a vector of the same length as the number of 
 ##' nonconstant rows in \code{X}. It will contain all integers from 0 to 
 ##' \eqn{N_g-1}{Ng-1}, where \eqn{N_g}{Ng} is the total number of \eqn{g} 
 ##' parameters. Each element of \code{gMap} specifies the group to which that 
@@ -21,8 +18,7 @@
 ##' If all columns belonging to a group are adjacent, \code{struc} can instead 
 ##' be used to compactly represent the groupings. \code{struc} is a vector of 
 ##' length \eqn{N_g}{Ng}. Each element specifies the number columns in the 
-##' group. \code{gMap} is thus the \code{\link{inverse.rle}} of \code{struc}, 
-##' minus 1.
+##' group. 
 ##' 
 ##' The vector \code{rscale} should be of length \eqn{N_g}{Ng}, and contain the 
 ##' prior scales of the standardized effects. See Rouder et al. (2012) for more 
@@ -45,25 +41,25 @@
 ##' @title Use ANOVA design matrix to compute Bayes factors or sample posterior
 ##' @param y vector of observations
 ##' @param X design matrix whose number of rows match \code{length(y)}.
-##' @param struc vector grouping the columns of \code{X} (see Details).
-##' @param gMap alternative way of grouping the columns of \code{X}
+##' @param gMap vector grouping the columns of \code{X} (see Details).
 ##' @param rscale a vector of prior scale(s) of appropriate length (see 
 ##'   Details).
 ##' @param iterations Number of Monte Carlo samples used to estimate Bayes 
 ##'   factor or posterior
 ##' @param progress  if \code{TRUE}, show progress with a text progress bar
 ##' @param callback callback function for third-party interfaces 
-##' @param gibbs if \code{TRUE}, return samples from the posterior instead of a 
-##'   Bayes factor
-##' @param ignoreCols if \code{NULL} and \code{gibbs=TRUE}, all parameter
+##' @param gibbs will be deprecated. See \code{posterior}
+##' @param posterior if \code{TRUE}, return samples from the posterior using 
+##' Gibbs sampling, instead of the Bayes factor
+##' @param ignoreCols if \code{NULL} and \code{posterior=TRUE}, all parameter
 ##'   estimates are returned in the MCMC object. If not \code{NULL}, a vector of
 ##'   length P-1 (where P is number of columns in the design matrix) giving which
 ##'   effect estimates to ignore in output
 ##' @param thin MCMC chain to every \code{thin} iterations. Default of 1 means 
-##'   no thinning. Only used if \code{gibbs=TRUE}
-##' @param method the integration method (only valid if \code{gibbs=TRUE}); one 
+##'   no thinning. Only used if \code{posterior=TRUE}
+##' @param method the integration method (only valid if \code{posterior=FALSE}); one 
 ##'   of "simple", "importance", "laplace", or "auto"
-##' @param continuous either FALSE is no continuous covariates are included, or 
+##' @param continuous either FALSE if no continuous covariates are included, or 
 ##'   a logical vector of length equal to number of columns of X indicating 
 ##'   which columns of the design matrix represent continuous covariates
 ##' @param noSample if \code{TRUE}, do not sample, instead returning NA. This is 
@@ -74,6 +70,8 @@
 ##'   with a proportional error estimate on the Bayes factor. Otherwise, an 
 ##'   object of class \code{mcmc}, containing MCMC samples from the posterior is
 ##'   returned.
+##' @note Argument \code{struc} has been deprecated. Use \code{gMap}, which is the \code{\link{inverse.rle}} of \code{struc}, 
+##' minus 1.
 ##' @export
 ##' @keywords htest
 ##' @author Richard D. Morey (\email{richarddmorey@@gmail.com}), Jeffery N. 
@@ -100,23 +98,24 @@
 ##' X <- cbind(group.column, subject.matrix)
 ##' 
 ##' ## (log) Bayes factor of full model against grand-mean only model
-##' bf.full <- nWayAOV(y = sleep$extra, X = X, struc = c(1,10), rscale=c(.5,1))
+##' bf.full <- nWayAOV(y = sleep$extra, X = X, gMap = c(0,rep(1,10)), rscale=c(.5,1))
 ##' exp(bf.full[['bf']])
 ##' 
 ##' ## Compare with lmBF result (should be about the same, give or take 1%)
 ##' bf.full2 <- lmBF(extra ~ group + ID, data = sleep, whichRandom = "ID")
 ##' bf.full2
 
-nWayAOV<- function(y, X, struc = NULL, gMap = NULL, rscale, iterations = 10000, progress = options()$BFprogress, callback = NULL, gibbs = FALSE, ignoreCols=NULL, thin=1, method="auto", continuous=FALSE, noSample = FALSE)
+nWayAOV<- function(y, X, gMap, rscale, iterations = 10000, progress = options()$BFprogress, callback = function(...) as.integer(0), gibbs = NULL, posterior = FALSE, ignoreCols=NULL, thin=1, method="auto", continuous=FALSE, noSample = FALSE)
 {  
   if(!is.numeric(y)) stop("y must be numeric.")  
   if(!is.numeric(X)) stop("X must be numeric.")  
+  if(!is.function(callback)) stop("Invalid callback.")  
   
-  y <- as.numeric(y)
-  
-  # Check thinning to make sure number is reasonable
-  if( (thin<1) | (thin>(iterations/3)) ) stop("MCMC thin parameter cannot be less than 1 or greater than iterations/3. Was:", thin)
-    
+  if(!is.null(gibbs)){
+    warning("Argument 'gibbs' to nWayAOV will soon be deprecated. Use 'posterior' instead.")
+    posterior = gibbs
+  }
+     
   N = length(y)
 	X = matrix( X, nrow=N )
   
@@ -133,157 +132,105 @@ nWayAOV<- function(y, X, struc = NULL, gMap = NULL, rscale, iterations = 10000,
       stop("Invalid gMap argument. length(gMap) must be the the same as the number of parameters (excluding intercept): ",sum(gMap)," != ",P)
     if( !all(0:max(gMap) %in% unique(gMap)) )
       stop("Invalid gMap argument: no index can be skipped.")
-    
     nGs = as.integer(max(gMap) + 1)
-  }else if(!is.null(struc)){
-    struc = unlist(struc)
-    if(sum(struc) != P)
-      stop("Invalid struc argument. sum(struc) must be the the same as the number of parameters (excluding intercept): ",sum(struc)," != ",P)
-    
-    nGs = length(struc)
-    gMap = as.integer(inverse.rle(list(values = (1:nGs)-1, lengths = struc)))
   }else{
-    stop("One of gMap or struc must be defined.")
+    stop("gMap must be defined.")
   }
   
   if(is.null(ignoreCols)) ignoreCols = rep(0,P)
   
-  a = rep(0.5,nGs)
-  if(length(rscale)==nGs){
-    b = rscale^2/2
-  }else{
+  if(length(rscale)!=nGs){
     stop("Length of rscale vector wrong. Was ", length(rscale), " and should be ", nGs,".")
   }
-  
-  nullLike = - ((N-1)/2)*log((N-1)*var(y))
-  
-  # What if we can use quadrature?
-  if(nGs==1 & !gibbs & all(!continuous)) 
-    return(singleGBayesFactor(y,X,rscale,gMap))
-  
-  Cy = y - mean(y)
-  CX = apply(X,2,function(v) v - mean(v))
-  
+      
   # Rearrange design matrix if continuous columns are included
   # We will undo this later if chains have to be returned
   if(!identical(continuous,FALSE)){
     if(all(continuous)){
       #### If all covariates are continuous, we want to use Gaussian quadrature.
-      #warning("All covariates are continuous: using Gaussian quadrature.")
-      if(gibbs){
-        chains = linearReg.Gibbs(y, X, iterations = iterations, 
-                                 rscale = rscale, progress = progress, callback = callback)
-        return(chains)
-      }else{
+        Cy = matrix(y - mean(y), ncol=1)
+        CX = t(t(X) - colMeans(X))
         R2 = t(Cy)%*%CX%*%solve(t(CX)%*%CX)%*%t(CX)%*%Cy / (t(Cy)%*%Cy)
         bf = linearReg.R2stat(N=N,p=ncol(CX),R2=R2,rscale=rscale)  
         return(bf)
-      }
-    } 
+    }
     if(length(continuous) != P) stop("argument continuous must have same length as number of predictors")
     if(length(unique(gMap[continuous]))!=1) stop("gMap for continuous predictors don't all point to same g value")
+    
+    # Sort chains so that continuous covariates are together, and first
     sortX = order(!continuous)
     revSortX = order(sortX)
     X = X[,sortX]
-    CX = CX[,sortX]
     gMap = gMap[sortX]
     ignoreCols = ignoreCols[sortX]
+    continuous = continuous[sortX]
     incCont = sum(continuous)
-    if(incCont>1){
-      X[,1:incCont] = CX[,1:incCont]
-      priorX = (t(CX[,1:incCont]) %*% CX[,1:incCont])/N
-    }else{
-      priorX = sum(CX[,1]^2)/N
-    }
   }else{
-    incCont = 0
-    priorX = 1
+    revSortX = sortX = 1:ncol(X)
+    incCont = as.integer(0)
   }
   
-  XtCX = t(CX) %*% CX
-  #XtCy = t(CX) %*% C %*% as.matrix(y,cols=1)
-  XtCy = t(CX) %*% as.matrix(y,cols=1)
-  ytCy = var(y)*(N-1)
+  # What if we can use quadrature?
+  if(nGs==1 & !posterior & all(!continuous)) 
+    return(singleGBayesFactor(y,X,rscale,gMap, incCont))
   
-
-	
-  
-  ####### Progress bar stuff
-	if(is.null(callback) | !is.function(callback)) {
-    callback = function(...){
-      return(as.integer(0))
-    }
-	}
-	if(progress & !noSample){
-		pb = txtProgressBar(min = 0, max = 100, style = 3) 
-	}else{ 
-		pb=NULL 
-	}
-	
-  pbFun = function(samps){ 
-    if(progress){
-    	percent = as.integer(round(samps / iterations * 100))
-    	setTxtProgressBar(pb, percent)
-    }
-  }
-  ############ End progress bar stuff
-  
-  
-  if(gibbs){ # Create chains
-    Z = cbind(1,X)
-    ZtZ = t(Z)%*%Z
-    Zty = t(Z)%*%matrix(y,ncol=1)
-    
-    # set up for not outputing some parameters
-    nOutputPars = sum(1-ignoreCols)
-    ignoreColsExtend = c(0,ignoreCols,0,rep(0,nGs))
-      
-    # should we sample?
-    
-    if(noSample){
-      chains = matrix(NA,nOutputPars + 2 + nGs,2)
-    }else{  
-      chains = .Call("RGibbsNwayAov", 
-                   as.integer(iterations), y, Z, ZtZ, priorX, Zty, as.integer(N), 
-                   as.integer(P), as.integer(nGs), as.integer(gMap), rscale, as.integer(incCont),
-                   as.integer(ignoreColsExtend), as.integer(thin),
-                   as.integer(iterations/100*progress), pbFun, callback, new.env(), 
-                   package="BayesFactor")
-    
-      dim(chains) <- c(nOutputPars + 2 + nGs, as.integer(iterations) %/% as.integer(thin))
-    }
-    chains = mcmc(t(chains))  
-    # Unsort the chains if we had continuous covariates
-    if(incCont){
-      # Account for ignored columns when resorting
-      revSort = 1+order(sortX[!ignoreCols])
-      chains[,1 + 1:nOutputPars] = chains[,revSort]
-      labels = c("mu",paste("beta",1:P,sep="_")[!ignoreCols[revSortX]],"sig2",paste("g",1:nGs,sep="_"))
-    }else{
-      labels = c("mu",paste("beta",1:P,sep="_")[!ignoreCols],"sig2",paste("g",1:nGs,sep="_"))
-    }
-    colnames(chains) = labels
-    retVal = chains
+  if(posterior)
+    return(nWayAOV.Gibbs(y, X, gMap, rscale, iterations, incCont, sortX, revSortX, progress, ignoreCols, thin, continuous, noSample, callback))
  
-  }else if(noSample){
-    retVal = c(bf = NA, properror=NA)
-  }else{# Compute Bayes factor
+  if(!noSample){
     if(method %in% c("simple","importance","auto")){
-      retVal = doNwaySampling(method, y, X, rscale, nullLike, 
-                   as.integer(iterations), XtCX, priorX, XtCy, ytCy, 
-                   as.integer(N), as.integer(P), as.integer(nGs), 
-                   as.integer(gMap), a, b, as.integer(incCont), progress, pbFun, callback)
+      return(doNwaySampling(method, y, X, rscale, 
+                   iterations, gMap, incCont, progress, callback))
     }else if(method=="laplace"){
-      bf = laplaceAOV(y,X,rscale,gMap,priorX,incCont)
-      properror=NA
-      retVal = c(bf = bf, properror=properror)
-      if(inherits(pb,"txtProgressBar")) close(pb);
-      return(retVal)
+      bf = laplaceAOV(y,X,rscale,gMap,incCont)
+      return(c(bf = bf, properror=NA))
     }else{  
       stop("Unknown method specified.")
     }
   }
   
-  if(inherits(pb,"txtProgressBar")) close(pb);
-  return(retVal)
+  return(c(bf = NA, properror=NA))
 }
+
+
+nWayAOV.Gibbs = function(y, X, gMap, rscale, iterations, incCont, sortX, revSortX, progress, ignoreCols, thin, continuous, noSample, callback)
+{
+  P = ncol(X)
+  nGs = as.integer( max(gMap) + 1 )
+
+  # Check thinning to make sure number is reasonable
+  if( (thin<1) | (thin>(iterations/3)) ) stop("MCMC thin parameter cannot be less than 1 or greater than iterations/3. Was:", thin)
+  
+  if(!identical(continuous,FALSE)){
+    if(all(continuous)){
+        chains = linearReg.Gibbs(y, X, iterations = iterations, 
+                                 rscale = rscale, progress = progress, callback = callback)
+        return(chains)
+    } 
+    if(length(continuous) != P) stop("argument continuous must have same length as number of predictors")
+    if(length(unique(gMap[continuous]))!=1) stop("gMap for continuous predictors don't all point to same g value")
+  }
+  
+  nOutputPars = sum(1-ignoreCols)
+  
+  if(noSample){ # Return structure of chains
+    chains = matrix(NA,2,nOutputPars + 2 + nGs)
+  }else{  
+    chains = jzs_Gibbs(iterations, y, cbind(1,X), rscale, 1, gMap, table(gMap), incCont, FALSE, 
+                       as.integer(ignoreCols), as.integer(thin), as.logical(progress), callback, 1)
+  }
+  chains = mcmc(chains)  
+  
+  # Unsort the chains if we had continuous covariates
+  if(incCont){
+    # Account for ignored columns when resorting
+    revSort = 1+order(sortX[!ignoreCols])
+    chains[,1 + 1:nOutputPars] = chains[,revSort]
+    labels = c("mu",paste("beta",1:P,sep="_")[!ignoreCols[revSortX]],"sig2",paste("g",1:nGs,sep="_"))
+  }else{
+    labels = c("mu",paste("beta",1:P,sep="_")[!ignoreCols],"sig2",paste("g",1:nGs,sep="_"))
+  }
+  colnames(chains) = labels
+  return(chains)
+}
+
